/*****************************************************************************
 *                DigCC: a minimal C compiler for DigMIPS                    *
 ***************************************************************************** 
 *                                                                           *
 * Copyright (c) 2012 Christophe Alias                                       *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU General Public License as published by the     *
 * Free Software Foundation; either version 2 of the License, or (at your    * 
 * option) any later version.                                                *
 *                                                                           *
 * This program is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General  *
 * Public License for more details.                                          *
 *                                                                           *
 * You should have received a copy of the GNU General Public License along   *
 * with this program; if not, write to the Free Software Foundation, Inc.,   *
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.                 *
 *                                                                           *
 * Written by Christophe Alias, Christophe.Alias@ens-lyon.fr                 *
 *                                                                           *
 *****************************************************************************/
#ifndef __ALLOCATION_H
#define __ALLOCATION_H

#include "InterferenceGraph.h"

#include "BitVector.h"

#include <map>

using namespace std;


enum temporary_t
  {
    REGISTER,
    STACK
  };


class Temporary
{
 public:
  int is_register;
  int reg;

  int is_stack;
  int shift;

  Temporary(temporary_t type, int);
};


class Allocation
{
 public:
  map<int,Temporary*> assignment;
  int nb_temporary; //#stack temporary

  Allocation(InterferenceGraph*, int K, map<int,int> precolor);

 private:
  int only_precolored_nodes(BitVector* nodes);
  int degree(InterferenceGraph* g, BitVector* nodes, int s);
  int remaining_color(InterferenceGraph* g, BitVector* nodes, int s, int K);

  void chaitin(InterferenceGraph* g, BitVector* nodes, int K);
};


//Pretty printer
ostream& operator <<(ostream& sout, Temporary& p);
ostream& operator <<(ostream& sout, Allocation& p);

#endif
